@@ -1,4 +1,4 @@
-from .guardpoint import GuardPoint, GuardPointError
+from .guardpoint import GuardPoint, GuardPointError, GuardPointUnauthorized
 from .guardpoint_dataclasses import SortAlgorithm, Cardholder, Card, Area, SecurityGroup, CardholderPersonalDetail, \
-    CardholderCustomizedField
+    CardholderCustomizedField, CardholderType
 from .guardpoint_async import GuardPointAsync
