@@ -4,7 +4,7 @@ import validators
 from ._odata_filter import _compose_filter, _compose_select, _compose_expand
 from ._str_match_algo import fuzzy_match
 from .guardpoint_dataclasses import Cardholder, SortAlgorithm, Area
-from .guardpoint_error import GuardPointError
+from .guardpoint_error import GuardPointError, GuardPointUnauthorized
 from .guardpoint_utils import GuardPointResponse
 
 
@@ -23,13 +23,14 @@ class CardholdersAPI:
             GuardPointResponse.check_odata_body_structure(json_body)
 
         if code != 204:  # HTTP NO_CONTENT
-            try:
-                if 'error' in json_body:
-                    raise GuardPointError(json_body['error'])
-                else:
-                    raise GuardPointError(str(code))
-            except Exception:
-                raise GuardPointError(str(code))
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Cardholder Not Found")
+            else:
+                raise GuardPointError(f"{error_msg}")
 
         return True
 
@@ -83,21 +84,20 @@ class CardholdersAPI:
         }
 
         code, json_body = self.gp_json_query("PATCH", headers=headers, url=(url + url_query_params), json_body=ch)
-        # Check response body is formatted correctly
-        if json_body:
-            GuardPointResponse.check_odata_body_structure(json_body)
 
         if code != 204:  # HTTP NO_CONTENT
-            if 'error' in json_body:
-                raise GuardPointError(json_body['error'])
-            elif 'message' in json_body:
-                raise GuardPointError(json_body['message'])
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Cardholder Not Found")
             else:
-                raise GuardPointError(str(code))
+                raise GuardPointError(f"{error_msg}")
 
         return True
 
-    def new_card_holder(self, cardholder: Cardholder, overwrite_cardholder=False):
+    def new_card_holder(self, cardholder: Cardholder, changed_only=False):
 
         # url = "/odata/API_Cardholders/CreateFullCardholder"
         url = "/odata/API_Cardholders"
@@ -108,7 +108,11 @@ class CardholdersAPI:
             'IgnoreNonEditable': ''
         }
 
-        ch = cardholder.dict(editable_only=True, changed_only=True)
+        if changed_only:
+            ch = cardholder.dict(editable_only=True, changed_only=True)
+        else:
+            ch = cardholder.dict(editable_only=True)
+
 
         # When using CreateFullCardholder
         # body = {'cardholder': ch}
@@ -126,6 +130,14 @@ class CardholdersAPI:
             if cardholder.cardholderCustomizedField:
                 self.update_custom_fields(cardholder_uid=new_cardholder.uid,
                                           customFields=cardholder.cardholderCustomizedField)
+            if cardholder.cards:
+                if isinstance(cardholder.cards, list):
+                    for card in cardholder.cards:
+                        if validators.uuid(card.uid):
+                            self.update_card(card)
+                        else:
+                            card.cardholderUID = new_cardholder.uid
+                            self.new_card(card)
 
             return self._get_card_holder(new_cardholder.uid)
 
@@ -134,12 +146,14 @@ class CardholdersAPI:
                 raise GuardPointError(
                     f'{json_body["errorMessages"][0]["message"]}-{json_body["errorMessages"][0]["other"]}')
         else:
-            if "errorMessages" in json_body:
-                raise GuardPointError(json_body["errorMessages"][0]["other"])
-            elif "error" in json_body:
-                raise GuardPointError(json_body["error"]['message'])
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Cardholder Not Found")
             else:
-                raise GuardPointError(str(code))
+                raise GuardPointError(f"{error_msg}")
 
     def get_card_holder(self,
                         uid: str = None,
@@ -159,15 +173,18 @@ class CardholdersAPI:
 
         code, json_body = self.gp_json_query("GET", url=(url + url_query_params))
         # Check response body is formatted correctly
-        if json_body:
-            GuardPointResponse.check_odata_body_structure(json_body)
+        # if json_body:
+        #    GuardPointResponse.check_odata_body_structure(json_body)
 
         if code != 200:
-            if isinstance(json_body, dict):
-                if 'error' in json_body:
-                    raise GuardPointError(json_body['error'])
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Cardholder Photo Not Found")
             else:
-                raise GuardPointError(str(code))
+                raise GuardPointError(f"{error_msg}")
 
         return json_body['value'][0]['photo']
 
@@ -178,7 +195,7 @@ class CardholdersAPI:
         url = "/odata/API_Cardholders"
         url_query_params = "(" + uid + ")?" \
                                        "$expand=" \
-                                       "cardholderType($select=typeName)," \
+                                       "cardholderType," \
                                        "cards," \
                                        "cardholderCustomizedField," \
                                        "cardholderPersonalDetail," \
@@ -187,15 +204,21 @@ class CardholdersAPI:
 
         code, json_body = self.gp_json_query("GET", url=(url + url_query_params))
         # Check response body is formatted correctly
-        if json_body:
-            GuardPointResponse.check_odata_body_structure(json_body)
+        # if json_body:
+        #    GuardPointResponse.check_odata_body_structure(json_body)
+
+        if code == 404:  # Not Found
+            return None
 
         if code != 200:
-            if isinstance(json_body, dict):
-                if 'error' in json_body:
-                    raise GuardPointError(json_body['error'])
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Cardholder Not Found")
             else:
-                raise GuardPointError(str(code))
+                raise GuardPointError(f"{error_msg}")
 
         return Cardholder(json_body['value'][0])
 
@@ -203,7 +226,17 @@ class CardholdersAPI:
                          filter_expired: bool = False, cardholder_type_name: str = None,
                          sort_algorithm: SortAlgorithm = SortAlgorithm.SERVER_DEFAULT, threshold: int = 75,
                          count: bool = False, earliest_last_pass: datetime = None,
-                         select_ignore_list: list = None, select_include_list: list = None):
+                         select_ignore_list: list = None, select_include_list: list = None,
+                         **cardholder_kwargs):
+
+        if offset is None:
+            offset = 0
+
+        # Filter arguments which have to exact match
+        match_args = dict()
+        for k, v in cardholder_kwargs.items():
+            if hasattr(Cardholder, k):
+                match_args[k] = v
 
         url = "/odata/API_Cardholders"
 
@@ -211,14 +244,14 @@ class CardholdersAPI:
                                      areas=areas,
                                      filter_expired=filter_expired,
                                      cardholder_type_name=cardholder_type_name,
-                                     earliest_last_pass=earliest_last_pass)
+                                     earliest_last_pass=earliest_last_pass,
+                                     exact_match=match_args)
 
         select_str = _compose_select(select_ignore_list, select_include_list)
 
         expand_str = _compose_expand(select_ignore_list, select_include_list)
 
         url_query_params = ("?" + select_str + expand_str + filter_str)
-        # url_query_params = ("?" + filter_str)
 
         if count:
             url_query_params += "$count=true&$top=0"
@@ -228,17 +261,18 @@ class CardholdersAPI:
 
         code, json_body = self.gp_json_query("GET", url=(url + url_query_params))
         # Check response body is formatted correctly
-        if json_body:
-            GuardPointResponse.check_odata_body_structure(json_body)
+        # if json_body:
+        #    GuardPointResponse.check_odata_body_structure(json_body)
 
         if code != 200:
-            if json_body:
-                if 'error' in json_body:
-                    raise GuardPointError(f"{json_body['error']} - ({code})")
-                else:
-                    raise GuardPointError(f"No error message - ({code})")
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Cardholders Not Found")
             else:
-                raise GuardPointError(f"No body - ({code})")
+                raise GuardPointError(f"{error_msg}")
 
         if count:
             return json_body['@odata.count']
