@@ -1,5 +1,6 @@
+from .guardpoint_utils import GuardPointResponse
 from .guardpoint_dataclasses import SecurityGroup
-from .guardpoint_error import GuardPointError
+from .guardpoint_error import GuardPointError, GuardPointUnauthorized
 
 
 class SecurityGroupsAPI:
@@ -10,10 +11,14 @@ class SecurityGroupsAPI:
         code, json_body = self.gp_json_query("GET", url=(url + url_query_params))
 
         if code != 200:
-            if isinstance(json_body, dict):
-                if 'error' in json_body:
-                    raise GuardPointError(json_body['error'])
-            raise GuardPointError(str(code))
+            error_msg = GuardPointResponse.extract_error_msg(json_body)
+
+            if code == 401:
+                raise GuardPointUnauthorized(f"Unauthorized - ({error_msg})")
+            elif code == 404:  # Not Found
+                raise GuardPointError(f"Security Group Not Found")
+            else:
+                raise GuardPointError(f"{error_msg}")
 
         # Check response body is formatted as expected
         if not isinstance(json_body, dict):
