@@ -25,6 +25,7 @@ class Observable:
 
 def sanitise_args(obj: Observable, args, kwargs):
     kwarg_dict = dict()
+    obj.changed_attributes = set()
 
     for arg in args:
         if isinstance(arg, dict):
@@ -64,7 +65,11 @@ class Card(Observable):
         for property_name in card_dict:
             if isinstance(card_dict[property_name], (str, type(None), bool, int)):
                 setattr(self, property_name, card_dict[property_name])
-                self.add_observer(property_name)
+
+        # Monitor Changes
+        for k, v in asdict(self).items():
+            if isinstance(v, (str, type(None), bool, int)):
+                self.add_observer(k)
 
     def _remove_non_changed(self, ch: dict):
         for key, value in list(ch.items()):
@@ -269,8 +274,9 @@ class CardholderPersonalDetail(Observable):
             if isinstance(v, (str, type(None), bool, int)):
                 self.add_observer(k)
 
-    def dict(self, changed_only=False):
+    def dict(self, editable_only=False, changed_only=False, non_empty_only=False):
         ch = dict()
+
         for k, v in asdict(self).items():
             if isinstance(v, (list, dict, bool, int)):
                 ch[k] = v
@@ -278,7 +284,10 @@ class CardholderPersonalDetail(Observable):
                 pass
                 # ch[k] = None
             elif isinstance(v, str):
-                if len(v) > 0:
+                if non_empty_only:
+                    if len(v) > 0:
+                        ch[k] = str(v)
+                else:
                     ch[k] = str(v)
             else:
                 pass
@@ -286,6 +295,9 @@ class CardholderPersonalDetail(Observable):
         if changed_only:
             ch = self._remove_non_changed(ch)
 
+        if editable_only:
+            ch = self._remove_non_editable(ch)
+
         return ch
 
     def _remove_non_changed(self, ch: dict):
@@ -294,10 +306,23 @@ class CardholderPersonalDetail(Observable):
                 ch.pop(key)
         return ch
 
+    @staticmethod
+    def _remove_non_editable(ch: dict):
+        if 'uid' in ch:
+            ch.pop('uid')
+        return ch
+
 
 @dataclass
 class CardholderType:
-    typeName: str
+    uid: str = ""
+    typeName: str = ""
+    defaultBPTemplate: str = ""
+
+    def __init__(self, cardholder_type_dict: dict):
+        for property_name in cardholder_type_dict:
+            if isinstance(cardholder_type_dict[property_name], (str, type(None), bool, int)):
+                setattr(self, property_name, cardholder_type_dict[property_name])
 
     def dict(self):
         return {k: str(v) for k, v in asdict(self).items()}
@@ -357,7 +382,10 @@ class Cardholder(Observable):
                 if property_name == "cards":
                     setattr(self, property_name, [])
                     for card_entry in cardholder_dict[property_name]:
-                        self.cards.append(Card(card_entry))
+                        if isinstance(card_entry, Card):
+                            self.cards.append(card_entry)
+                        else:
+                            self.cards.append(Card(card_entry))
                 else:
                     setattr(self, property_name, cardholder_dict[property_name])
 
@@ -375,7 +403,7 @@ class Cardholder(Observable):
                 if property_name == "securityGroup":
                     self.securityGroup = SecurityGroup(cardholder_dict[property_name])
                 if property_name == "cardholderType":
-                    self.cardholderType = CardholderType(typeName=cardholder_dict[property_name]['typeName'])
+                    self.cardholderType = CardholderType(cardholder_dict[property_name])
                 if property_name == "cardholderPersonalDetail":
                     self.cardholderPersonalDetail = CardholderPersonalDetail(cardholder_dict[property_name])
                 if property_name == "cardholderCustomizedField":
@@ -389,7 +417,6 @@ class Cardholder(Observable):
             if isinstance(v, (str, type(None), bool, int)):
                 self.add_observer(k)
 
-
     def to_search_pattern(self):
         pattern = ""
         if self.firstName:
@@ -418,26 +445,30 @@ class Cardholder(Observable):
                 else:
                     print(f"\t{attribute_name:<25}" + str(attribute))
 
-    def dict(self, editable_only=False, changed_only=False):
+    def dict(self, editable_only=False, changed_only=False, non_empty_only=False):
         ch = dict()
+
         for k, v in asdict(self).items():
             if isinstance(v, (list, dict, bool, int)):
                 ch[k] = v
             elif isinstance(v, type(None)):
-                pass
-                #ch[k] = None
+                if not non_empty_only:
+                    ch[k] = None
             elif isinstance(v, str):
-                if len(v) > 0:
+                if non_empty_only:
+                    if len(v) > 0:
+                        ch[k] = str(v)
+                else:
                     ch[k] = str(v)
             else:
                 pass
 
-        if editable_only:
-            ch = self._remove_non_editable(ch)
-
         if changed_only:
             ch = self._remove_non_changed(ch)
 
+        if editable_only:
+            ch = self._remove_non_editable(ch)
+
         return ch
 
     def _remove_non_changed(self, ch: dict):
